@@ -3,23 +3,25 @@
 # Author: Lesserkuma (github.com/lesserkuma)
 
 import time, copy, math, struct
-from .Util import ANSI, dprint, bitswap
-from . import Util
+from .Util import dprint, bitswap
 
 class Flashcart:
 	CONFIG = {}
 	COMMAND_SET = None
 	CART_WRITE_FNCPTR = None
 	CART_READ_FNCPTR = None
+	CART_POWERCYCLE_FNCPTR = None
 	PROGRESS_FNCPTR = None
 	SECTOR_COUNT = 0
 	SECTOR_POS = 0
 	SECTOR_MAP = None
 	CFI = None
 
-	def __init__(self, config={}, cart_write_fncptr=None, cart_read_fncptr=None, progress_fncptr=None):
+	def __init__(self, config=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, progress_fncptr=None):
+		if config is None: config = {}
 		self.CART_WRITE_FNCPTR = cart_write_fncptr
 		self.CART_READ_FNCPTR = cart_read_fncptr
+		self.CART_POWERCYCLE_FNCPTR = cart_powercycle_fncptr
 		self.PROGRESS_FNCPTR = progress_fncptr
 		self.CONFIG = config
 		if "command_set" in config:
@@ -37,11 +39,12 @@ class Flashcart:
 				length = 1
 		return self.CART_READ_FNCPTR(address, length)
 	
-	def CartWrite(self, commands, sram=False):
+	def CartWrite(self, commands, flashcart=True, sram=False):
+		if "command_set" in self.CONFIG and self.CONFIG["command_set"] == "DMG-MBC5-32M-FLASH": flashcart = False
 		for command in commands:
 			address = command[0]
 			value = command[1]
-			self.CART_WRITE_FNCPTR(address, value, flashcart=True, sram=sram)
+			self.CART_WRITE_FNCPTR(address, value, flashcart=flashcart, sram=sram)
 
 	def GetCommandSetType(self):
 		return self.CONFIG["_command_set"].upper()
@@ -58,12 +61,12 @@ class Flashcart:
 	def GetMBC(self):
 		if (self.CONFIG["type"].upper() == "AGB") or ("mbc" not in self.CONFIG): return False
 		mbc = self.CONFIG["mbc"]
-		if mbc == 1: mbc = 0x03
-		elif mbc == 2: mbc = 0x06
-		elif mbc == 3: mbc = 0x13
-		elif mbc == 5: mbc = 0x1B
-		elif mbc == 6: mbc = 0x20
-		elif mbc == 7: mbc = 0x22
+		#if mbc == 1: mbc = 0x03
+		#elif mbc == 2: mbc = 0x06
+		#elif mbc == 3: mbc = 0x13
+		#elif mbc == 5: mbc = 0x1B
+		#elif mbc == 6: mbc = 0x20
+		#elif mbc == 7: mbc = 0x22
 		return mbc
 
 	def FlashCommandsOnBank1(self):
@@ -110,6 +113,10 @@ class Flashcart:
 		if "write_pin" not in self.CONFIG: return False
 		return (self.CONFIG["write_pin"] == "WR+RESET")
 
+	def GetFlashSize(self):
+		if "flash_size" not in self.CONFIG: return False
+		return self.CONFIG["flash_size"]
+
 	def GetBufferSize(self):
 		if "buffer_size" in self.CONFIG:
 			return self.CONFIG["buffer_size"]
@@ -119,11 +126,11 @@ class Flashcart:
 			else:
 				cfi = self.ReadCFI()
 				if cfi is False:
-					print("CFI error") 
+					print("CFI ERROR: Couldn’t retrieve buffer size from the cartridge.")
 					return False
 			if not "buffer_size" in cfi: return False
 			buffer_size = cfi["buffer_size"]
-			dprint("Buffer size was read from Common Flash Interface (CFI) data:", cfi["buffer_size"])
+			dprint("Buffer size was read from CFI data:", cfi["buffer_size"])
 			self.CONFIG["buffer_size"] = buffer_size
 			return buffer_size
 		else:
@@ -139,20 +146,27 @@ class Flashcart:
 			self.CartWrite(self.CONFIG["commands"]["unlock"])
 			time.sleep(0.001)
 
-	def Reset(self, full_reset=False):
+	def Reset(self, full_reset=False, max_address=0x2000000):
 		#dprint(full_reset, "reset_every" in self.CONFIG)
-		if full_reset and "reset_every" in self.CONFIG:
+		if full_reset and "power_cycle" in self.CONFIG:
+			self.CART_POWERCYCLE_FNCPTR()
+			time.sleep(0.001)
+		elif full_reset and "reset_every" in self.CONFIG:
 			for j in range(0, self.CONFIG["flash_size"], self.CONFIG["reset_every"]):
+				if j >= max_address: break
+				dprint("reset_every @ 0x{:X}".format(j))
 				for command in self.CONFIG["commands"]["reset"]:
 					self.CartWrite([[j, command[1]]])
-					time.sleep(0.001)
+					time.sleep(0.01)
 		elif "reset" in self.CONFIG["commands"]:
 			self.CartWrite(self.CONFIG["commands"]["reset"])
 			time.sleep(0.001)
 	
 	def VerifyFlashID(self):
 		if "read_identifier" not in self.CONFIG["commands"]: return False
+		if len(self.CONFIG["flash_ids"]) == 0: return False
 		self.Reset()
+		rom = list(self.CartRead(0, len(self.CONFIG["flash_ids"][0])))
 		self.Unlock()
 		self.CartWrite(self.CONFIG["commands"]["read_identifier"])
 		time.sleep(0.001)
@@ -160,7 +174,10 @@ class Flashcart:
 		self.Reset()
 		dprint("Flash ID: {:s}".format(' '.join(format(x, '02X') for x in cart_flash_id)))
 		verified = True
-		if cart_flash_id not in self.CONFIG["flash_ids"]:
+		if (rom == cart_flash_id):
+			dprint("ROM data matched Flash ID response.")
+			verified = False
+		elif cart_flash_id not in self.CONFIG["flash_ids"]:
 			dprint("This Flash ID does not exist in flashcart handler file.")
 			verified = False
 		return (verified, cart_flash_id)
@@ -208,11 +225,11 @@ class Flashcart:
 			else:
 				cfi = self.ReadCFI()
 				if cfi is False:
-					print("CFI error") 
+					print("CFI ERROR: Couldn’t retrieve sector size map from the cartridge.")
 					return False
 			sector_size = cfi["erase_sector_blocks"]
 			if cfi["tb_boot_sector_raw"] == 0x03: sector_size.reverse()
-			dprint("Sector map was read from Common Flash Interface (CFI) data:", cfi["erase_sector_blocks"])
+			dprint("Sector size map was read from CFI data:", cfi["erase_sector_blocks"])
 			self.CONFIG["sector_size"] = sector_size
 			return sector_size
 		else:
@@ -240,7 +257,7 @@ class Flashcart:
 							self.CartWrite([[addr, sr_data]])
 					self.CartRead(addr, 2) # dummy read (fixes some bootlegs)
 					wait_for = struct.unpack("<H", self.CartRead(addr, 2))[0]
-					dprint("Status Register Check: 0x{:X} & 0x{:X} == 0x{:X}? {:s}".format(wait_for, self.CONFIG["commands"]["chip_erase_wait_for"][i][2], data, str(wait_for == data)))
+					dprint("Status Register Check: 0x{:X} & 0x{:X} == 0x{:X}? {:s}".format(wait_for, self.CONFIG["commands"]["chip_erase_wait_for"][i][2], data, str((wait_for & self.CONFIG["commands"]["chip_erase_wait_for"][i][2]) == data)))
 					wait_for = wait_for & self.CONFIG["commands"]["chip_erase_wait_for"][i][2]
 					if wait_for == data: break
 					time.sleep(0.5)
@@ -256,6 +273,7 @@ class Flashcart:
 		#time_start = time.time()
 		#if progress_fnc is not None: progress_fnc({"action":"ERASE", "time_start":time_start, "abortable":False})
 		if "sector_erase" not in self.CONFIG["commands"]: return False
+		if "sector_size" not in self.CONFIG: return False
 		for i in range(0, len(self.CONFIG["commands"]["sector_erase"])):
 			addr = self.CONFIG["commands"]["sector_erase"][i][0]
 			data = self.CONFIG["commands"]["sector_erase"][i][1]
@@ -306,7 +324,7 @@ class Flashcart:
 			except:
 				dprint("Warning: Sector map is smaller than expected.")
 				self.SECTOR_POS -= 1
-				self.CONFIG["sector_size"][self.SECTOR_POS][0]
+				#self.CONFIG["sector_size"][self.SECTOR_POS][0]
 			return sector_size
 		else:
 			return self.CONFIG["sector_size"]
@@ -436,7 +454,7 @@ class CFI:
 			s += "Buffered write: {:s}\n".format(str(info["buffer_write"]))
 		if info["chip_erase"]: s += "Chip erase: {:d}–{:d} ms\n".format(info["chip_erase_time_avg"], info["chip_erase_time_max"])
 		if info["sector_erase"]: s += "Sector erase: {:d}–{:d} ms\n".format(info["sector_erase_time_avg"], info["sector_erase_time_max"])
-		if info["tb_boot_sector"] is not False: s += "Sector order: {:s}\n".format(str(info["tb_boot_sector"]))
+		if info["tb_boot_sector"] is not False: s += "Sector flags: {:s}\n".format(str(info["tb_boot_sector"]))
 		pos = 0
 		oversize = False
 		s = s[:-1]
