@@ -5,60 +5,69 @@
 import time, datetime, struct, math, hashlib
 from dateutil.relativedelta import relativedelta
 from .RomFileDMG import RomFileDMG
-from .Util import ANSI, dprint
+from .Util import dprint
 from . import Util
 
 class DMG_MBC:
 	MBC_ID = 0
 	CART_WRITE_FNCPTR = None
 	CART_READ_FNCPTR = None
+	CART_POWERCYCLE_FNCPTR = None
 	CLK_TOGGLE_FNCPTR = None
-	ROM_BANK_SIZE = 0
-	RAM_BANK_SIZE = 0
+	ROM_BANK_SIZE = 0x4000
+	RAM_BANK_SIZE = 0x2000
 	ROM_BANK_NUM = 0
 	CURRENT_ROM_BANK = 0
 
-	def __init__(self, args={}, cart_write_fncptr=None, cart_read_fncptr=None, clk_toggle_fncptr=None):
+	def __init__(self, args=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, clk_toggle_fncptr=None):
+		if args is None: args = {}
 		if "mbc" in args: self.MBC_ID = args["mbc"]
-		if "rom_banks" in args: self.ROM_BANK_NUM = args["rom_banks"]
+		if "rom_banks" in args:
+			self.ROM_BANK_NUM = args["rom_banks"]
+		elif "rom_size" in args:
+			self.ROM_BANK_NUM = math.ceil(args["rom_size"] / self.ROM_BANK_SIZE)
 		self.CART_WRITE_FNCPTR = cart_write_fncptr
 		self.CART_READ_FNCPTR = cart_read_fncptr
+		self.CART_POWERCYCLE_FNCPTR = cart_powercycle_fncptr
 		self.CLK_TOGGLE_FNCPTR = clk_toggle_fncptr
-		self.ROM_BANK_SIZE = 0x4000
-		self.RAM_BANK_SIZE = 0x2000
 
-	def GetInstance(self, args={}, cart_write_fncptr=None, cart_read_fncptr=None, clk_toggle_fncptr=None):
+	def GetInstance(self, args=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, clk_toggle_fncptr=None):
+		if args is None: args = {}
 		mbc_id = args["mbc"]
 		if mbc_id in (0x01, 0x02, 0x03):						# 0x01:'MBC1', 0x02:'MBC1+SRAM', 0x03:'MBC1+SRAM+BATTERY',
-			return DMG_MBC1(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC1(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0x06:									# 0x06:'MBC2+SRAM+BATTERY',
-			return DMG_MBC2(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC2(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id in (0x10, 0x13):							# 0x10:'MBC3+RTC+SRAM+BATTERY', 0x13:'MBC3+SRAM+BATTERY',
-			return DMG_MBC3(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC3(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id in (0x19, 0x1B, 0x1C, 0x1E):				# 0x19:'MBC5', 0x1B:'MBC5+SRAM+BATTERY', 0x1C:'MBC5+RUMBLE', 0x1E:'MBC5+RUMBLE+SRAM+BATTERY',
-			return DMG_MBC5(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC5(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0x20:									# 0x20:'MBC6+FLASH+SRAM+BATTERY',
-			return DMG_MBC6(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC6(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0x22:									# 0x22:'MBC7+ACCELEROMETER+EEPROM',
-			return DMG_MBC7(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC7(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id in (0x101, 0x103):							# 0x101:'MBC1M', 0x103:'MBC1M+SRAM+BATTERY',
-			return DMG_MBC1M(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MBC1M(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id in (0x0B, 0x0D):							# 0x0B:'MMM01',  0x0D:'MMM01+SRAM+BATTERY',
-			return DMG_MMM01(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_MMM01(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0xFC:									# 0xFC:'GBD+SRAM+BATTERY',
-			return DMG_GBD(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_GBD(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0x105:									# 0x105:'G-MMC1+SRAM+BATTERY',
-			return DMG_GMMC1(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_GMMC1(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0x104:									# 0x104:'M161',
-			return DMG_M161(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_M161(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0xFF:									# 0xFF:'HuC-1+IR+SRAM+BATTERY',
-			return DMG_HuC1(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_HuC1(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0xFE:									# 0xFE:'HuC-3+RTC+SRAM+BATTERY',
-			return DMG_HuC3(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_HuC3(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		elif mbc_id == 0xFD:									# 0xFD:'TAMA5+RTC+EEPROM'
-			return DMG_TAMA5(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			return DMG_TAMA5(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+		elif mbc_id == 0x201:									# 0x201:'256M Multi Cart',
+			return DMG_Unlicensed_256M(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+		elif mbc_id == 0x202:									# 0x202:'Wisdom Tree Mapper',
+			return DMG_Unlicensed_WisdomTree(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 		else:
-			self.__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
+			self.__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=clk_toggle_fncptr)
 			return self
 	
 	def CartRead(self, address, length=0):
@@ -75,13 +84,13 @@ class DMG_MBC:
 			if delay is not False: time.sleep(delay)
 
 	def GetName(self):
-		return "Unknown MBC"
+		return "Unknown MBC {:d}".format(self.MBC_ID)
 
 	def GetFullName(self):
 		try:
 			return Util.DMG_Header_Mapper[self.MBC_ID]
 		except:
-			return "Unknown MBC"
+			return "Unknown MBC {:d}".format(self.MBC_ID)
 
 	def GetROMBank(self):
 		return self.CURRENT_ROM_BANK
@@ -201,37 +210,60 @@ class DMG_MBC3(DMG_MBC):
 	def EnableRAM(self, enable=True):
 		dprint(self.GetName(), "|", enable)
 		commands = [
-			[ 0x6000, 0x01 if enable else 0x00 ],
+			#[ 0x6000, 0x01 if enable else 0x00 ],
 			[ 0x0000, 0x0A if enable else 0x00 ],
 		]
 		self.CartWrite(commands)
 	
 	def HasRTC(self):
 		dprint(self.GetName())
-		self.EnableRAM(enable=True)
-		ram1 = self.CartRead(0xA000, 0x10)
-		self.LatchRTC()
-		self.CartWrite([ [0x4000, 0x08] ])
-		ram2 = self.CartRead(0xA000, 0x10)
+		if self.MBC_ID != 16: return False
 		self.EnableRAM(enable=False)
-		#return (self.MBC_ID == 0x10)
-		return (ram1 != ram2)
+		self.EnableRAM(enable=True)
+		#self.CartWrite([ [0x4000, 0x08] ])
+		self.LatchRTC()
+
+		skipped = True
+		for i in range(0x08, 0x0D):
+			self.CLK_TOGGLE_FNCPTR(60)
+			self.CartWrite([ [0x4000, i] ])
+			data = self.CartRead(0xA000, 0x800)
+			if data[0] in (0, 0xFF): continue
+			skipped = False
+			if data != bytearray([data[0]] * 0x800): return False
+		return skipped is False
+
+		#ram1 = self.CartRead(0xA000, 0x10)
+		#ram2 = ram1
+		#t1 = time.time()
+		#t2 = 0
+		#while t2 < (t1 + 1):
+		#	self.LatchRTC()
+		#	ram2 = self.CartRead(0xA000, 0x10)
+		#	if ram1 != ram2: break
+		#	t2 = time.time()
+		#dprint("RTC_S {:02X} != {:02X}?".format(ram1[0], ram2[0]), ram1 != ram2)
+		#time.sleep(0.1)
+		#return (ram1 != ram2)
 
 	def GetRTCBufferSize(self):
 		return 0x30
 
 	def LatchRTC(self):
-		self.EnableRAM(enable=True)
-		self.CartWrite([
-			[ 0x6000, 0 ],
-			[ 0x6000, 1 ],
-		])
-		self.CLK_TOGGLE_FNCPTR(5)
+		self.CLK_TOGGLE_FNCPTR(60)
+		self.CartWrite([ [ 0x0000, 0x0A ] ])
+		time.sleep(0.01)
+		self.CLK_TOGGLE_FNCPTR(60)
+		self.CartWrite([ [ 0x6000, 0x00 ] ])
+		self.CLK_TOGGLE_FNCPTR(60)
+		self.CartWrite([ [ 0x6000, 0x01 ] ])
+		time.sleep(0.01)
 
 	def ReadRTC(self):
-		self.EnableRAM(enable=True)
+		#self.EnableRAM(enable=True)
 		buffer = bytearray()
 		for i in range(0x08, 0x0D):
+			self.CLK_TOGGLE_FNCPTR(60)
 			self.CartWrite([ [0x4000, i] ])
 			buffer.extend(struct.pack("<I", self.CartRead(0xA000)))
 		buffer.extend(buffer) # copy
@@ -239,15 +271,12 @@ class DMG_MBC3(DMG_MBC):
 		# Add timestamp of backup time
 		ts = int(time.time())
 		buffer.extend(struct.pack("<Q", ts))
-
-		dstr = ' '.join(format(x, '02X') for x in buffer)
-		dprint("[{:02X}] {:s}".format(int(len(dstr)/3) + 1, dstr))
-
+		
 		return buffer
 
 	def WriteRTC(self, buffer, advance=False):
 		dprint(buffer)
-		self.LatchRTC()
+		#self.LatchRTC()
 		if advance:
 			try:
 				dt_now = datetime.datetime.fromtimestamp(time.time())
@@ -274,16 +303,18 @@ class DMG_MBC3(DMG_MBC):
 					if timestamp_then < timestamp_now:
 						dt_then = datetime.datetime.fromtimestamp(timestamp_then)
 						dt_buffer1 = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(2000, 1, 1, 0, 0, 0), "%Y-%m-%d %H:%M:%S")
-						dt_buffer2 = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(2000, 1, 1, hours, minutes, seconds), "%Y-%m-%d %H:%M:%S")
+						dt_buffer2 = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(2000, 1, 1, hours % 24, minutes % 60, seconds % 60), "%Y-%m-%d %H:%M:%S")
 						dt_buffer2 += datetime.timedelta(days=days)
 						rd = relativedelta(dt_now, dt_then)
 						dt_new = dt_buffer2 + rd
-						#print(dt_then, dt_now, dt_buffer2, dt_new, sep="\n")
+						dprint(dt_then, dt_now, dt_buffer1, dt_buffer2, dt_new, sep="\n")
 						seconds = dt_new.second
 						minutes = dt_new.minute
 						hours = dt_new.hour
-						temp = dt_new - dt_buffer1
-						days = temp.days
+						#temp = dt_new - dt_buffer1
+						#days = temp.days
+						temp = datetime.date.fromtimestamp(timestamp_now) - datetime.date.fromtimestamp(timestamp_then)
+						days = temp.days + days
 						if days >= 512:
 							carry = True
 							days = days % 512
@@ -299,14 +330,39 @@ class DMG_MBC3(DMG_MBC):
 			except Exception as e:
 				print("Couldn’t update the RTC register values\n", e)
 		
+		dprint("New values: RTC_S=0x{:02X}, RTC_M=0x{:02X}, RTC_H=0x{:02X}, RTC_DL=0x{:02X}, RTC_DH=0x{:02X}".format(buffer[0x00], buffer[0x04], buffer[0x08], buffer[0x0C], buffer[0x10]))
+
+		# Unlock and latch RTC
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0x0000, 0x0A ] ])
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0x6000, 0x00 ] ])
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0x6000, 0x01 ] ])
+		time.sleep(0.01)
+
+		# Halt RTC
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0x4000, 0x0C ] ])
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0xA000, 0x40 ] ])
+		time.sleep(0.1)
+		
 		# Write to registers
 		for i in range(0x08, 0x0D):
-			self.CartWrite([ [0x4000, i] ])
+			self.CLK_TOGGLE_FNCPTR(50)
+			self.CartWrite([ [ 0x4000, i ] ])
+			self.CLK_TOGGLE_FNCPTR(50)
 			data = struct.unpack("<I", buffer[(i-8)*4:(i-8)*4+4])[0] & 0xFF
-			dprint("Writing 0x{:X} to 0x{:X}".format(data, i))
-			self.CartWrite([ [0xA000, data] ])
-			self.CLK_TOGGLE_FNCPTR(5)
-		self.LatchRTC()
+			self.CartWrite([ [ 0xA000, data ] ])
+		time.sleep(0.1)
+
+		# Latch RTC
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0x6000, 0x00 ] ])
+		self.CLK_TOGGLE_FNCPTR(50)
+		self.CartWrite([ [ 0x6000, 0x01 ] ])
+		time.sleep(0.1)
 
 class DMG_MBC5(DMG_MBC):
 	def GetName(self):
@@ -338,8 +394,9 @@ class DMG_MBC5(DMG_MBC):
 		return (start_address, self.ROM_BANK_SIZE)
 
 class DMG_MBC6(DMG_MBC):
-	def __init__(self, args={}, cart_write_fncptr=None, cart_read_fncptr=None, clk_toggle_fncptr=None):
-		super().__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=None)
+	def __init__(self, args=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, clk_toggle_fncptr=None):
+		if args is None: args = {}
+		super().__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=None)
 		self.ROM_BANK_SIZE = 0x2000
 		self.RAM_BANK_SIZE = 0x1000
 		self.ROM_BANK_NUM = 128
@@ -375,7 +432,7 @@ class DMG_MBC6(DMG_MBC):
 		start_address = 0x4000
 		return (start_address, self.ROM_BANK_SIZE)
 
-	def GetRAMBanks(self, size): # 0x108000
+	def GetRAMBanks(self, ram_size): # 0x108000
 		return 8 + 128
 
 	def SelectBankRAM(self, index):
@@ -420,7 +477,7 @@ class DMG_MBC6(DMG_MBC):
 		self.CartWrite(cmds)
 		self.SelectBankFlash(self.GetROMBank())
 		self.CartWrite([[ 0x4000, 0x30 ]])
-		while True: # TODO: error handling
+		while True:
 			sr = self.CartRead(0x4000)
 			dprint("Status Register Check: 0x{:X} == 0x80? {:s}".format(sr, str(sr == 0x80)))
 			if sr == 0x80: break
@@ -666,14 +723,17 @@ class DMG_M161(DMG_MBC):
 	def GetName(self):
 		return "M161"
 
-	def __init__(self, args={}, cart_write_fncptr=None, cart_read_fncptr=None, clk_toggle_fncptr=None):
-		super().__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, clk_toggle_fncptr=None)
+	def __init__(self, args=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, clk_toggle_fncptr=None):
+		if args is None: args = {}
 		self.ROM_BANK_SIZE = 0x8000
-		self.ROM_BANK_NUM = 8
+		super().__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=None)
 	
 	def ResetBeforeBankChange(self, index):
 		return True
 	
+	#def GetROMSize(self):
+	#	return self.ROM_BANK_SIZE * math.floor(self.ROM_BANK_NUM / 2)
+
 	def SelectBankROM(self, index):
 		dprint(self.GetName(), "|", index)
 		commands = [
@@ -682,7 +742,7 @@ class DMG_M161(DMG_MBC):
 		self.CartWrite(commands)
 		return (0, 0x8000)
 
-class DMG_HuC1(DMG_MBC1):
+class DMG_HuC1(DMG_MBC5):
 	def GetName(self):
 		return "HuC-1"
 
@@ -763,11 +823,6 @@ class DMG_HuC3(DMG_MBC):
 					timestamp_then = struct.unpack("<Q", buffer[-8:])[0]
 					timestamp_now = int(time.time())
 					dprint(hours, minutes, days)
-					#debug
-					#timestamp_now = 4102441140 # 2099-12-23 23:59:00
-					timestamp_now = time.time() + (3600 * 24 * 4095) + (3600 * 9)
-					dt_now = datetime.datetime.fromtimestamp(timestamp_now)
-					#debug
 					if timestamp_then < timestamp_now:
 						dt_then = datetime.datetime.fromtimestamp(timestamp_then)
 						dt_buffer1 = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(1, 1, 1, 0, 0, 0), "%Y-%m-%d %H:%M:%S")
@@ -775,11 +830,13 @@ class DMG_HuC3(DMG_MBC):
 						dt_buffer2 += datetime.timedelta(days=days)
 						rd = relativedelta(dt_now, dt_then)
 						dt_new = dt_buffer2 + rd
-						#print(dt_then, dt_now, dt_buffer1, dt_buffer2, dt_new, sep="\n")
+						dprint(dt_then, dt_now, dt_buffer1, dt_buffer2, dt_new, sep="\n")
 						minutes = dt_new.minute
 						hours = dt_new.hour
-						temp = dt_new - dt_buffer1
-						days = temp.days
+						#temp = dt_new - dt_buffer1
+						#days = temp.days
+						temp = datetime.date.fromtimestamp(timestamp_now) - datetime.date.fromtimestamp(timestamp_then)
+						days = temp.days + days
 						dprint(minutes, hours, days)
 				
 				total_minutes = 60 * hours + minutes
@@ -941,7 +998,7 @@ class DMG_TAMA5(DMG_MBC):
 					timestamp_now = int(time.time())
 					if timestamp_then < timestamp_now:
 						dt_then = datetime.datetime.fromtimestamp(timestamp_then)
-						dt_buffer = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(1, months, days, hours, minutes, seconds), "%Y-%m-%d %H:%M:%S")
+						dt_buffer = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(1, months % 12, days % 31, hours % 24, minutes % 60, seconds % 60), "%Y-%m-%d %H:%M:%S")
 						rd = relativedelta(dt_now, dt_then)
 						dt_new = dt_buffer + rd
 						#print(dt_then, dt_now, dt_buffer, dt_new, sep="\n")
@@ -995,6 +1052,81 @@ class DMG_TAMA5(DMG_MBC):
 	def WriteWithCSPulse(self):
 		return True
 
+class DMG_Unlicensed_256M(DMG_MBC5):
+	def GetName(self):
+		return "256M Multi Cart"
+	
+	def SelectBankROM(self, index):
+		dprint(self.GetName(), "|", index)
+
+		flash_bank = math.floor(index / 512)
+		if index >= 512:
+			index -= (512 * flash_bank)
+
+		if index == 0:
+			self.CART_POWERCYCLE_FNCPTR()
+			commands = [
+				[ 0x7000, 0x00 ],
+				[ 0x7001, 0x00 ],
+				[ 0x7002, 0x90 + flash_bank ]
+			]
+			self.CartWrite(commands, delay=0.1)
+		
+		commands = [
+			[ 0x2100, index & 0xFF ],
+			[ 0x3000, ((index >> 8) & 0xFF) ],
+		]
+		
+		start_address = 0 if index == 0 else 0x4000
+		self.CartWrite(commands)
+
+		return (start_address, self.ROM_BANK_SIZE)
+
+	def SelectBankRAM(self, index):
+		dprint(self.GetName(), "|", index)
+
+		flash_bank = math.floor(index / 0x10)
+		
+		if index % 4 == 0:
+			self.EnableRAM(enable=False)
+			self.CART_POWERCYCLE_FNCPTR()
+			commands = [
+				[ 0x7000, (0x40 * math.floor(index / 4)) & 0xFF ],
+				[ 0x7001, 0xC0 ],
+				[ 0x7002, 0x90 + flash_bank ]
+			]
+			self.CartWrite(commands, delay=0.01)
+			self.EnableRAM(enable=True)
+			dprint(hex(index), hex(0x90 + flash_bank), hex((0x40 * math.floor(index / 4)) & 0xFF))
+		
+		commands = [
+			[ 0x4000, index % 4 ]
+		]
+		start_address = 0
+		self.CartWrite(commands)
+
+		return (start_address, self.RAM_BANK_SIZE)
+
+class DMG_Unlicensed_WisdomTree(DMG_MBC):
+	def GetName(self):
+		return "Wisdom Tree"
+
+	def __init__(self, args=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, clk_toggle_fncptr=None):
+		if args is None: args = {}
+		self.ROM_BANK_SIZE = 0x8000
+		super().__init__(args=args, cart_write_fncptr=cart_write_fncptr, cart_read_fncptr=cart_read_fncptr, cart_powercycle_fncptr=cart_powercycle_fncptr, clk_toggle_fncptr=None)
+	
+	#def GetROMSize(self):
+	#	return self.ROM_BANK_SIZE * math.floor(self.ROM_BANK_NUM / 2)
+
+	def SelectBankROM(self, index):
+		dprint(self.GetName(), "|", index)
+		commands = [
+			[ index, 0 ]
+		]
+		self.CartWrite(commands)
+		return (0, 0x8000)
+
 
 class AGB_GPIO:
 	CART_WRITE_FNCPTR = None
@@ -1018,9 +1150,11 @@ class AGB_GPIO:
 	RTC_WRITE_ALARM = 0x68
 	RTC_READ_ALARM = 0x69
 
-	def __init__(self, args={}, cart_write_fncptr=None, cart_read_fncptr=None, clk_toggle_fncptr=None):
+	def __init__(self, args=None, cart_write_fncptr=None, cart_read_fncptr=None, cart_powercycle_fncptr=None, clk_toggle_fncptr=None):
+		if args is None: args = {}
 		self.CART_WRITE_FNCPTR = cart_write_fncptr
 		self.CART_READ_FNCPTR = cart_read_fncptr
+		self.CART_POWERCYCLE_FNCPTR = cart_powercycle_fncptr
 		self.CLK_TOGGLE_FNCPTR = clk_toggle_fncptr
 		if "rtc" in args: self.RTC = args["rtc"]
 	
@@ -1151,12 +1285,13 @@ class AGB_GPIO:
 		
 		# Add timestamp of backup time
 		ts = int(time.time())
-		buffer.extend(b'\x01') # 24h mode (TODO: read from status?)
+		buffer.extend(b'\x01') # 24h mode (TODO: read from cart?)
 		buffer.extend(struct.pack("<Q", ts))
 
 		dstr = ' '.join(format(x, '02X') for x in buffer)
 		dprint("[{:02X}] {:s}".format(int(len(dstr)/3) + 1, dstr))
 		
+		# Digits are BCD (Binary Coded Decimal)
 		#[07] 00 01 27 05 06 30 20
 		#[07] 00 01 27 05 06 30 28
 		# "27 days, 06:51:55"
@@ -1192,7 +1327,7 @@ class AGB_GPIO:
 					#debug
 					if timestamp_then < timestamp_now:
 						dt_then = datetime.datetime.fromtimestamp(timestamp_then)
-						dt_buffer = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(years + 2000, months, days, hours, minutes, seconds), "%Y-%m-%d %H:%M:%S")
+						dt_buffer = datetime.datetime.strptime("{:04d}-{:02d}-{:02d} {:02d}:{:02d}:{:02d}".format(years + 2000, months % 12, days % 31, hours % 60, minutes % 60, seconds % 60), "%Y-%m-%d %H:%M:%S")
 						rd = relativedelta(dt_now, dt_then)
 						dt_new = dt_buffer + rd
 						#print(dt_then, dt_now, dt_buffer, dt_new, sep="\n")
