@@ -2,7 +2,7 @@
 # FlashGBX
 # Author: Lesserkuma (github.com/lesserkuma)
 
-import hashlib, re, zlib, sys, string
+import hashlib, re, zlib, string
 
 class RomFileAGB:
 	ROMFILE_PATH = None
@@ -24,26 +24,26 @@ class RomFileAGB:
 			self.ROMFILE = bytearray(f.read())
 	
 	def CalcChecksumHeader(self, fix=False):
-		buffer = self.ROMFILE
 		checksum = 0
 		for i in range(0xA0, 0xBD):
-			checksum = checksum - buffer[i]
+			checksum = checksum - self.ROMFILE[i]
 		checksum = (checksum - 0x19) & 0xFF
 		
-		if fix: buffer[0x14D] = checksum
+		if fix: self.ROMFILE[0xBD] = checksum
 		return checksum
 	
 	def CalcChecksumGlobal(self):
-		buffer = self.ROMFILE
-		return (zlib.crc32(buffer) & 0xffffffff)
+		return (zlib.crc32(self.ROMFILE) & 0xFFFFFFFF)
 	
-	def FixChecksums(self):
+	def FixHeader(self):
 		self.CalcChecksumHeader(True)
+		return self.ROMFILE[0:0x200]
 	
 	def GetHeader(self):
 		buffer = self.ROMFILE
 		data = {}
-		data["empty"] = (self.ROMFILE == bytearray([buffer[0]] * len(buffer)))
+		data["empty"] = (buffer[0x04:0xA0] == bytearray([buffer[0x04]] * 0x9C))
+		data["empty_nocart"] = (buffer == bytearray([0x00] * len(buffer)))
 		data["logo_correct"] = hashlib.sha1(buffer[0x04:0xA0]).digest() == bytearray([ 0x17, 0xDA, 0xA0, 0xFE, 0xC0, 0x2F, 0xC3, 0x3C, 0x0F, 0x6A, 0xBB, 0x54, 0x9A, 0x8B, 0x80, 0xB6, 0x61, 0x3B, 0x48, 0xEE ])
 		game_title = bytearray(buffer[0xA0:0xAC]).decode("ascii", "replace")
 		game_title = re.sub(r"(\x00+)$", "", game_title)
@@ -90,6 +90,3 @@ class RomFileAGB:
 			data["dacs_8m"] = True
 		
 		return data
-
-	def GetData(self):
-		return self.ROMFILE
